from django.core.cache import caches
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from .serializers import TimeInTransitRequestSerializer
from .tnt import fetch_estimated_arrival_times
from . import settings


cache = caches[settings.UPS_CACHE_NAME]


class TimeInTransitView(APIView):
    """ Uses UPS Time In Transit API to estimate delivery date
    `postal_code` is required for a location.
    `country_code`, `city`, `state_province_code` are optional

    Example json request

    ```
    {
        "ship_to": {
            "city": "New York",
            "state_province_code": "NY",
            "postal_code": "10031"
        }
    }
    ```
    """

    def _build_cache_key(self, data):
        key = 'ups_tnt_1_%s_%s_%s' % (
            data.get('ship_from'),
            data.get('ship_to'),
            data.get('pickup_date')
        )
        # Remove memcached unfriendly chars
        return key.translate({ord(i): None for i in '[]:() '})

    def post(self, request, format=None):
        serializer = TimeInTransitRequestSerializer(data=request.data)
        if not serializer.is_valid():
            return Response(
                serializer.errors, status=status.HTTP_400_BAD_REQUEST)

        cache_key = self._build_cache_key(serializer.validated_data)
        response_data = cache.get(cache_key)
        if response_data is None:
            response_data = fetch_estimated_arrival_times(
                **serializer.validated_data)
            cache.set(cache_key, response_data, 60 * 60)  # Cache for 1 hour

        return Response(response_data)



from . import settings
from .exceptions import UPSException
import datetime
import requests
import logging


logger = logging.getLogger(__name__)
ups_url = 'https://wwwcie.ups.com/json/TimeInTransit'


def prepare_location(location):
    ups_data = {'Address': {}}
    if location.get('city'):
        ups_data['Address']['City'] = location['city']
    if location.get('state_province_code'):
        ups_data['Address']['StateProvinceCode'] = location['state_province_code']
    if location.get('postal_code'):
        ups_data['Address']['PostalCode'] = location['postal_code']
    if location.get('country_code'):
        ups_data['Address']['CountryCode'] = location['country_code']
    return ups_data


def fetch_estimated_arrival_times(ship_from=None, ship_to=None, pickup_date=None):

    if ship_from:
        ship_from = prepare_location(ship_from)
    else:
        ship_from = settings.UPS_DEFAULT_SHIP_FROM

    if ship_to:
        ship_to = prepare_location(ship_to)
    else:
        ship_to = settings.UPS_DEFAULT_SHIP_TO

    if pickup_date is None:
        if isinstance(settings.UPS_BUFFER_DAYS, int):
            pickup_date = datetime.date.today() + datetime.timedelta(days=settings.UPS_BUFFER_DAYS)
        if callable(settings.UPS_BUFFER_DAYS):
            pickup_date = datetime.date.today() + datetime.timedelta(days=settings.UPS_BUFFER_DAYS())
    pickup_date = pickup_date.strftime('%Y%m%d')

    ups_data = {
        "Security": {
            "UsernameToken": {
                "Username": settings.UPS_USERNAME,
                "Password": settings.UPS_PASSWORD,
            },
            "UPSServiceAccessToken": {
                "AccessLicenseNumber": settings.UPS_ACCESS_LICENSE_NUMBER
            }
        },
        "TimeInTransitRequest": {
            "Request": {
                "TransactionReference": {
                    "CustomerContext": "TNT Request with Documents only Indicator"
                }
            },
            "ShipFrom": ship_from,
            "ShipTo": ship_to,
            "Pickup": {
                "Date": pickup_date
            },
            "ShipmentWeight": settings.UPS_DEFAULT_SHIPMENT_WEIGHT,
            "InvoiceLineTotal": settings.UPS_DEFAULT_INVOICE,
        }
    }

    try:
        res = requests.post(ups_url, json=ups_data)
    except requests.exceptions.RequestException:
        raise UPSException('UPS api is not responding')

    res_data = res.json()

    # This occurs when UPS API is down
    if res_data.get('Error'):
        error_details = res_data['Error']['Description']
        logger.error(error_details)
        raise UPSException(error_details)

    # This occurs when UPS API is up, but returns some error.
    if res_data.get('Fault'):
        error_details = res_data['Fault']['detail']['Errors']['ErrorDetail']['PrimaryErrorCode']['Description']
        raise UPSException(error_details)

    try:
        services = res_data['TimeInTransitResponse']['TransitResponse']['ServiceSummary']
    except KeyError:
        raise UPSException("Unexpected response received from UPS")

    estimated = None
    for service in services:
        estimate_date = service['EstimatedArrival']['Arrival']['Date']
        estimate_time = service['EstimatedArrival']['Arrival']['Time']
        estimated = "%s%s" % (estimate_date, estimate_time)
        estimated = datetime.datetime.strptime(estimated, '%Y%m%d%H%M%S')
        service['EstimatedArrival']['DateTime'] = estimated.isoformat()
    return res_data['TimeInTransitResponse']['TransitResponse']



from celery import shared_task
from .exceptions import UPSException
from .tnt import fetch_estimated_arrival_times
import logging


logger = logging.getLogger(__name__)


@shared_task(bind=True, throws=(UPSException, ))
def task_fetch_estimated_arrival_times(**kwargs):
    return fetch_estimated_arrival_times(**kwargs)



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns
from .views import TimeInTransitView


urlpatterns = [
    url(r'^tnt/$', TimeInTransitView.as_view(), name='time-in-transit'),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from rest_framework import serializers


class LocationSerializer(serializers.Serializer):
    city = serializers.CharField(
        max_length=30, required=False, allow_blank=True, allow_null=True)
    state_province_code = serializers.CharField(
        max_length=30, required=False, allow_blank=True, allow_null=True)
    country_code = serializers.CharField(
        max_length=2, allow_blank=True, default="US")
    postal_code = serializers.CharField(max_length=10)


class TimeInTransitRequestSerializer(serializers.Serializer):
    ship_from = LocationSerializer(default=[])
    ship_to = LocationSerializer(default=[])
    pickup_date = serializers.DateField(default=None, allow_null=True)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def overridable(name, default=None, required=False):
    if required:
        if not hasattr(settings, name) or not getattr(settings, name):
            raise ImproperlyConfigured("%s must be defined in Django settings" % name)
    return getattr(settings, name, default)


UPS_DEFAULT_SHIP_FROM = overridable('UPS_DEFAULT_SHIP_FROM')
UPS_DEFAULT_SHIP_TO = overridable(
    'UPS_DEFAULT_SHIP_TO',
    default={
        "Address": {
            "CountryCode": "US",
        }
    }
)
UPS_USERNAME = overridable('UPS_USERNAME', required=True)
UPS_PASSWORD = overridable('UPS_PASSWORD', required=True)
UPS_ACCESS_LICENSE_NUMBER = overridable('UPS_ACCESS_LICENSE_NUMBER', required=True)
UPS_BUFFER_DAYS = overridable('UPS_BUFFER_DAYS', default=0)
UPS_DEFAULT_SHIPMENT_WEIGHT = overridable('UPS_DEFAULT_SHIPMENT_WEIGHT')
UPS_DEFAULT_INVOICE = overridable('UPS_DEFAULT_INVOICE')
UPS_TEST_LIVE = overridable('UPS_TEST_LIVE', default=False)
UPS_CACHE_NAME = overridable('UPS_CACHE_NAME', default="default")



#!/usr/bin/env python
import codecs
import os.path
from setuptools import setup
from versiontag import get_version, cache_git_tag


packages = [
    'ups_tnt',
]

setup_requires = [
    'versiontag>=1.0.3',
]

requires = [
    'Django>=1.8.11',
    'requests>=2.9.1',
    'djangorestframework>=3.3.3',
    'python-dateutil>=2.5.2',
]


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)

def read(fname):
    return codecs.open(fpath(fname), encoding='utf-8').read()

cache_git_tag()

setup(
    name='django-ups-tnt',
    description="Django wrapper around UPS Time In Transit JSON API",
    version='1.0.1',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    author='David Burke',
    author_email='dburke@thelabnyc.com',
    url='https://gitlab.com/thelabnyc/django-ups-tnt',
    license='ISC',
    packages=packages,
    install_requires=requires,
    setup_requires=setup_requires
)
